;
; Game concept for the Atari 2600 VCS
;
; Copyright 2001
; By Joe Grand, jgrand@mindspring.com
;
; dasm source.s -f3 -osource.bin to compile
;
; Tested with StellaX v1.1.3a

;
; TODO
;
; DONE! Draw score, logo, initial playfield [initial playfield]
; DONE! Enable movement up and down with joystick (easier for testing) [joystick]
; DONE! Display drive head on the right side of the screen [drivehead]
; Draw playfield, with moving drive head
; Generate sound when button is pressed
; Change joystick to paddle
; Draw new logo/title banner for bottom of the screen
;
; GAME FEATURES
; 
; Data latency buffer
; Bad blocks
; Various size data (fragmented)
;

;
; Clear memory and registers
; Set up variables
; Loop:
;    Do the vertical blank
;    Do game calculations
;    Draw screen
;    Do more calculations during overscan
;    Wait for next vertical blank
; End Loop.
;    
	processor 6502
	include vcs.h

; Allocate memory, PIA has 128 bytes of RAM ($80 - $FF), Stack starts from $FF

TempX1 		 = $80

DriveHeadYPos	 = $81  ;Vertical position of drive head
;GameNumber	 = $82	;Current game variation being played (0-7)
;Polynomial	 = $83	;Random number polynomial
Joystick0	 = $84	;Player 1's joystick value
ZColorScore	 = $86	;Score color
ZColorDriveHead  = $87
ZColorData	 = $88	;Data color
ZColorLine	 = $89	;Track seperator color
ZColorTrack	 = $8A	;Track color
ZColorBackground = $8B
TrackNumber	 = $8C	;Track currently being drawn

ScoreShape01Ptr	 = $8D	;Pointer to left score first digit
ScoreShape02Ptr	 = $8F 	;Pointer to left score second digit
;DriveHeadShapePtr = $90;Pointer to drive head bitmap (if we want multiple shapes)

;All variables from here to $FF are zero'd out every new game

Score  		 = $93	;Player score (BCD)


	org $F800 ; Limit ourselves to 2K like the old days

Start
;
; The 2600 powers up in a completely random state, except for the PC which
; is set to the location at $FFC.  Therefore the first thing we must do is
; to set everything up inside the 6502.
;
	SEI  ; Disable interrupts, if there are any.
	CLD  ; Clear BCD math bit.
	LDX  #$FF
	TXS  ; Set stack to beginning.

	; Loop to clear memory and TIA registers
	LDA #0
B1      STA 0,X
	DEX
	BNE B1

	; The above routine does not clear location 0, which is VSYNC

	JSR  GameInit ;Initial variable and register setup
MainLoop
	JSR  VerticalBlank ;Execute the vertical blank.
	JSR  CheckSwitches ;Check console switches.
	JSR  GameCalc      ;Do calculations during VBLANK
	JSR  DrawScreen    ;Draw the screen
	JSR  OverScan      ;Do more calculations during overscan
	JMP  MainLoop      ;Continue forever.


VerticalBlank  ;*********************** VERTICAL BLANK HANDLER
;
;Beginning of the frame - at the end of overscan.
;
	LDA  #2		;VBLANK was set at the beginning of overscan.
	STA  WSYNC  
	STA  WSYNC
	STA  WSYNC
	STA  VSYNC 	;Begin vertical sync
	STA  WSYNC 	;First line of VSYNC
	STA  WSYNC 	;Second line of VSYNC

	;Set the timer to go off just before the end of vertical blank space
	;37 scanlines * 76 cycles = 2812 cycles / 64 = approx. 44
	LDA  #44
	STA  TIM64T
;	
; End the VSYNC period.
;	
	LDA  #0
	STA  WSYNC ; Third line of VSYNC.
	STA  VSYNC ; (0)
	RTS  


CheckSwitches ;*************************** CONSOLE SWITCH HANDLER

	;LDA #0		;Clear collision latches
	;STA CXCLR	

   	LDA SWCHA       ;Read joystick 0
   	STA Joystick0   ;Store for later use


	LDA	SWCHB	;Read the switches
	LSR		;Shift the Reset switch
	ROR		;Rotate Reset to Negative flag and Carry has select
	
	; PAL/NTSC select check

	RTS

;
; Minimal game calculations, just to get the ball rolling.
;
GameCalc ;******************************* GAME CALCULATION ROUTINES

	LDA  #$30
	STA  CTRLPF	;No Reflect,No Score,Low Priority,8 pixel-wide ball

	; Get current Score and set ScoreShape pointers to point
	; to the right digits
	LDA	Score		;Get the score (BCD)
	AND	#$F0		;Mask it
	LSR			;$00,$08,$10,$18,$20
	BNE	NotZero
	LDA	#<Space		;Space character
NotZero	
	LDY	#0		;Needed?
	STA	ScoreShape01Ptr,Y	;Insert a space (or digit)
	LDA	Score			;Get the BCD number again
	AND	#$0F			;0-9
	ASL				;Mul by 8
	ASL
	ASL				;$00,$08,$10,$18,$20
	STA	ScoreShape02Ptr,Y	;Save the second digit

	; Set new vertical position of drive head based on joystick value
    	LDA Joystick0 	;Load the joystick switches
	ROL
	ROL
	BMI NotDown	;Joystick pushed down?
	LDX DriveHeadYPos
	CPX MaxYPos
	BEQ NoStick	;If DriveHeadYPos < MaxYPos, don't change
	DEC DriveHeadYPos
	DEC DriveHeadYPos
	JMP NoStick
NotDown
	ROL
	BMI NoStick	;Joystick pushed up?
	LDX DriveHeadYPos
	CPX MinYPos
	BEQ NoStick	;If DriveHeadYPos > MinYPos, don't change
	INC DriveHeadYPos
	INC DriveHeadYPos
NoStick
	RTS


DrawScreen ;**************************** SCREEN DRAWING ROUTINES

;
; Initialize display variables.
;
	LDA 	ZColorBackground
   	STA 	COLUBK  	;Background will be black.
	LDA	ZColorScore	;Set the score color (P0 and P1)
	STA	COLUP0
	STA	COLUP1
	LDX	#0
	STX	NUSIZ0		;One copy of player
	STX	NUSIZ1
	LDA	#24
	JSR	SetMotionRegsX	;Position digit #1
	INX			;X=1
	LDA	#32
	JSR	SetMotionRegsX	;Position digit #2
WaitVBlank
	LDA INTIM	;Loop until timer is done - wait for the proper scanline
			;(i.e. somewhere in the last line of VBLANK)
	BNE WaitVBlank 	;Whew! We're at the beginning of the frame
	STA WSYNC	;End the current scanline
	STA HMOVE	;Add horizontal motion to position sprites
	STA VBLANK  	;End the VBLANK period with a zero, enable video.
	STA WSYNC	;Wait 2 scanlines before drawing score
	STA WSYNC

ScanLoop ; ================================== SCANNING LOOP

;
; Display the scores (8 scan lines)
; 47 cycle loop
;
	LDY	#8-1			;Lines to display
DrawScore	
	STA	WSYNC			;3 Wait for sync
	STA	HMCLR			;3 Clear horizontal motion
	LDA	(ScoreShape01Ptr),Y	;5 Get the shape for player #0's score
	STA	GRP0			;3
	LDA	(ScoreShape02Ptr),Y	;5
	STA	GRP1			;3
	JSR	Waste18			;18 Wait for it to be displayed
	DEY				;2
	BPL	DrawScore		;3
	INY				;Set Y back to 0
	STY	GRP0			;Clear graphics
	STY	GRP1

	; Skip down a few lines
	; Do other calculations here if necessary, we have lots of time
	LDY	#10
DrawWait
	STA	WSYNC
	DEY
	BNE	DrawWait

	LDA	ZColorDriveHead
	STA	COLUP0
	LDX	#0
	LDA	#140
	JSR	SetMotionRegsX	;Set proper X axis
	STA	HMOVE

	LDA	ZColorLine ;Color of playfield (for testing)
	STA 	COLUBK

	LDY	MinYPos ;Set bounds on playfield
VertPos
	;Position drive head on proper Y axis
	CPY	DriveHeadYPos
	BNE	NoHead
	LDX	#16-1	;Size of drive head
DrawDriveHead
	LDA	HeadFrame,X 	;Get the shape of the drive head
	STA	GRP0
	STA	WSYNC	
	STA 	HMCLR
	DEY
	DEX
	BPL	DrawDriveHead
	INX		;X = 0
	STX	GRP0	;Disable graphics
	JMP	Done
NoHead

	STA	WSYNC
	DEY
Done
	CPY	#0
	BNE	VertPos


;	LDX	#10	;Init the lane count (10 lanes)
;	STX	TrackNumber
;
; Draw a track
;
;DrawDisk
;	LDA	ZColorLine ;Draw the top border of the disk
;	STA 	COLUBK
;	LDY	#15	;15 lines to draw
;	LDA	#0	;A = 0
;	STA	WSYNC
;DrawTrack	
;	LDA	ZColorTrack
;	STA	COLUBK
;	

	; This could be called as a function
	; Skip down a few lines
	; Do other calculations here if necessary, we have lots of time
;	LDA	ZColorLine ;Draw the bottom border of the disk
;	STA 	COLUBK
;	STA	WSYNC

	LDA	ZColorBackground	;Black line
	STA	COLUBK
	LDY	#10
DrawWait2
	STA	WSYNC
	DEY
	BNE	DrawWait2

;
; Prepare to draw the logo
; Waste 1 scan line
;
	STA	HMOVE
	NOP
	NOP
	NOP
	NOP
	STY	HMCLR			;Clear horizontal motion registers
	INY				;X = 1
	STY	NUSIZ0			;2 shapes, close together
	STY	NUSIZ1
	STA	RESP0			;Set the X coordinate
	STA	RESP1			
	LDA	#$10			;Move P1 one pixel to the left
	STA	HMP1
	LDA	ZColorData		;Score color
	STA	COLUP0
	STA	COLUP1

;
; Now draw the 8 lines
; Note : All the activision shapes end with a shape of zero,
; this way, I can assume the video display is pure black after the loop
; is finished.
;
	LDX	#8-1			;8 scan lines
DrawLogo
	STA	WSYNC			;3 Wait for sync
	STA	HMOVE			;3 Move the players (Only first time)
	LDA	Activision1,X		;4 Draw the shape
	STA	GRP0			;3
	LDA	Activision2,X		;4 Draw in player #1
	STA	GRP1			;3
	NOP				;2 Waste 4 cycles
	NOP				;2
	LDA	Activision4,X		;4 Preload the final 2 bytes
	TAY				;2
	LDA	Activision3,X		;4
	STA	GRP0			;3 Store in shape (Cycle 34)
	STY	GRP1			;3
	STA	HMCLR			;3 Clear horiz. motion for future loops
	DEX				;2 All done?
	BPL	DrawLogo		;3 Loop

;
; Clear all registers here to prevent any possible bleeding.
;
	LDA #2
	STA WSYNC  ;Finish this scanline.
	STA VBLANK ; Make TIA output invisible,
	; Now we need to worry about it bleeding when we turn
	; the TIA output back on.
	;LDY #0
	;STY PF0
	;STY PF1
	;STY PF1
	;STY GRP0
	;STY GRP1
	;STY ENAM0
	;STY ENAM1
	;STY ENABL
	RTS


OverScan   ;***************************** OVERSCAN CALCULATIONS
;
; For the Overscan routine, one might take the time to process such
; things as collisions.
;

	LDX #30
KillLines
	STA WSYNC
	DEX
	BNE KillLines
	RTS


GameInit   ;***************************** INITIAL VARIABLE & REGISTER SETUP
;
; GameInit could conceivably be called when the Select key is pressed,
; or some other event.
;
	;Load base colors 
	LDX	#6-1
NextColor
	LDA	BaseColors,X	;Get the colors
	STA	ZColorScore,X	;Save the adjusted colors
	CPX	#4
	;BPL	NotHardwareYet
	;STA	COLUP0,X	;Set the hardware defaults (P0, P1, PF, BK)
NotHardwareYet
	DEX
	BPL	NextColor

	; Set pointers to hard-coded data
	LDX	#4-1		;Only 2 pointers
	LDA	#>FontPage
NextPtr
	STA	ScoreShape01Ptr,X	;Init the high byte for video pointers
	DEX
	DEX
	BPL	NextPtr

	; Set data direction registers for paddles

	LDA #$69
	STA Score

	LDA MaxYPos		;Start with DriveHead position at bottom
	STA DriveHeadYPos	 

	RTS

;
; Set the horizontal motion registers
;

SetMotionRegsX	
	JSR	CalcXRegs	;Calculate the values
	STA	HMP0,X		;Save the horizontal motion register
	STA	WSYNC		;Sync to video
Wait	DEY			;Wait for cycle delay (5 cycles per)
	BPL	Wait
	STA	RESP0,X		;Hit the register for course adjustment
	RTS


;
; Given a requested X coord in A,
; return in A and Y and Horizonal offset and the cycle delay
;

CalcXRegs	
	CLC
	ADC	#46
	TAY
	AND	#$F	;Mask the 4 bit offset
	STA	TempX1	;Save temp
	TYA		;Get value again
	LSR		;Isolate the upper 4 bits
	LSR
	LSR
	LSR
	TAY		;Save as a WSYNC Delay
	CLC
	ADC	TempX1	;Mod 15
	CMP	#15
	BMI	NoExcess
	SBC	#15	;Remove one step
	INY		;5 more cycles
NoExcess	
	EOR	#7	;Negate horizontal offset
	ASL		;Move to upper 4 bits (Hardware)
Waste18	
	ASL		;2 Call to waste 18 cycles
	ASL		;2
Waste14	
	ASL		;2 Call to waste 14 cycles
Waste12	
	RTS		;6 Call to waste 12 cycles


;
; Must not cross page boundaries!!
; All shapes are upside down to allow decrementing by Y as both
; a counter and a shape index
;
	org $FF00 ; *********************** GRAPHICS DATA

;
; Numeric font
;

FontPage
Zero		.byte	%00111100 ; |  XXXX  |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%00111100 ; |  XXXX  |

One		.byte	%00111100 ; |  XXXX  |
		.byte	%00011000 ; |   XX   |
		.byte	%00011000 ; |   XX   |
		.byte	%00011000 ; |   XX   |
		.byte	%00011000 ; |   XX   |
		.byte	%00011000 ; |   XX   |
		.byte	%00111000 ; |  XXX   |
		.byte	%00011000 ; |   XX   |

Two		.byte	%01111110 ; | XXXXXX |
		.byte	%01100000 ; | XX     |
		.byte	%01100000 ; | XX     |
		.byte	%00111100 ; |  XXXX  |
		.byte	%00000110 ; |     XX |
		.byte	%00000110 ; |     XX |
		.byte	%01000110 ; | X   XX |
		.byte	%00111100 ; |  XXXX  |

Three		.byte	%00111100 ; |  XXXX  |
		.byte	%01000110 ; | X   XX |
		.byte	%00000110 ; |     XX |
		.byte	%00001100 ; |    XX  |
		.byte	%00001100 ; |    XX  |
		.byte	%00000110 ; |     XX |
		.byte	%01000110 ; | X   XX |
		.byte	%00111100 ; |  XXXX  |

Four		.byte	%00001100 ; |    XX  |
		.byte	%00001100 ; |    XX  |
		.byte	%00001100 ; |    XX  |
		.byte	%01111110 ; | XXXXXX |
		.byte	%01001100 ; | X  XX  |
		.byte	%00101100 ; |  X XX  |
		.byte	%00011100 ; |   XXX  |
		.byte	%00001100 ; |    XX  |

Five		.byte	%01111100 ; | XXXXX  |
		.byte	%01000110 ; | X   XX |
		.byte	%00000110 ; |     XX |
		.byte	%00000110 ; |     XX |
		.byte	%01111100 ; | XXXXX  |
		.byte	%01100000 ; | XX     |
		.byte	%01100000 ; | XX     |
		.byte	%01111110 ; | XXXXXX |

Six		.byte	%00111100 ; |  XXXX  |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%01111100 ; | XXXXX  |
		.byte	%01100000 ; | XX     |
		.byte	%01100010 ; | XX   X |
		.byte	%00111100 ; |  XXXX  |

Seven		.byte	%00011000 ; |   XX   |
		.byte	%00011000 ; |   XX   |
		.byte	%00011000 ; |   XX   |
		.byte	%00011000 ; |   XX   |
		.byte	%00001100 ; |    XX  |
		.byte	%00000110 ; |     XX |
		.byte	%01000010 ; | X    X |
		.byte	%01111110 ; | XXXXXX |

Eight		.byte	%00111100 ; |  XXXX  |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%00111100 ; |  XXXX  |
		.byte	%00111100 ; |  XXXX  |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%00111100 ; |  XXXX  |

Nine		.byte	%00111100 ; |  XXXX  |
		.byte	%01000110 ; | X   XX |
		.byte	%00000110 ; |     XX |
		.byte	%00111110 ; |  XXXXX |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%01100110 ; | XX  XX |
		.byte	%00111100 ; |  XXXX  |

Space		.byte   %00000000 ; |        |
		.byte   %00000000 ; |        |
		.byte   %00000000 ; |        |
		.byte   %00000000 ; |        |
		.byte   %00000000 ; |        |
		.byte   %00000000 ; |        |
		.byte   %00000000 ; |        |
		.byte   %00000000 ; |        |

;
; Activision logo bitmap, 32 x 8
;

Activision1	.byte	%00000000 ; |        |
		.byte	%10101101 ; |X X XX X|
		.byte	%10101001 ; |X X X  X|
		.byte	%11101001 ; |XXX X  X|
		.byte	%10101001 ; |X X X  X|
		.byte	%11101101 ; |XXX XX X|
		.byte	%01000001 ; | X     X|
		.byte	%00001111 ; |    XXXX|

Activision2	.byte	%00000000 ; |        |
		.byte	%01010000 ; | X X    |
		.byte	%01011000 ; | X XX   |
		.byte	%01011100 ; | X XXX  |
		.byte	%01010110 ; | X X XX |
		.byte	%01010011 ; | X X  XX|
		.byte	%00010001 ; |   X   X|
		.byte	%11110000 ; |XXXX    |

Activision3	.byte	%00000000 ; |        |
		.byte	%10111010 ; |X XXX X |
		.byte	%10001010 ; |X   X X |
		.byte	%10111010 ; |X XXX X |
		.byte	%10100010 ; |X X   X |
		.byte	%00111010 ; |  XXX X |
		.byte	%10000000 ; |X       |
		.byte	%11111110 ; |XXXXXXX |

Activision4	.byte	%00000000 ; |        |
		.byte	%11101001 ; |XXX X  X|
		.byte	%10101011 ; |X X X XX|
		.byte	%10101111 ; |X X XXXX|
		.byte	%10101101 ; |X X XX X|
		.byte	%11101001 ; |XXX X  X|
		.byte	%00000000 ; |        |
		.byte	%00000000 ; |        |

HeadFrame
       		;.byte 	%00111111 ; |  XXXXXX|
      		;.byte 	%00111111 ; |  XXXXXX|
      		;.byte 	%00011100 ; |   XXX  |
       		;.byte 	%11111100 ; |XXXXXX  |
       		;.byte 	%00011100 ; |   XXX  |
       		;.byte 	%00111111 ; |  XXXXXX|
       		;.byte 	%00111111 ; |  XXXXXX|
		;.byte 	%00000000 ; |        |

		.byte 	%00000000 ; |        |
		.byte 	%00111111 ; |  XXXXXX|
      		.byte 	%00111111 ; |  XXXXXX|
		.byte 	%00111111 ; |  XXXXXX|
      		.byte 	%00111111 ; |  XXXXXX|
      		.byte 	%00011100 ; |   XXX  |
       		.byte 	%00011100 ; |   XXX  |
       		.byte 	%11111100 ; |XXXXXX  |
		.byte 	%11111100 ; |XXXXXX  |
       		.byte 	%00011100 ; |   XXX  |
       		.byte 	%00011100 ; |   XXX  |
       		.byte 	%00111111 ; |  XXXXXX|
       		.byte 	%00111111 ; |  XXXXXX|
		.byte 	%00111111 ; |  XXXXXX|
       		.byte 	%00111111 ; |  XXXXXX|
		.byte 	%00000000 ; |        |

DataFrame	.byte	%01100110 ; | XX  XX |
		.byte	%11111110 ; |XXXXXXX |
		.byte	%11001111 ; |XX  XXXX|
		.byte	%10110011 ; |X XX  XX|
		.byte	%10110011 ; |X XX  XX|
		.byte	%10110011 ; |X XX  XX|
		.byte	%10110011 ; |X XX  XX|
		.byte	%11001111 ; |XX  XXXX|
		.byte	%11111110 ; |XXXXXXX |
		.byte	%01100110 ; | XX  XX |
		.byte	%00000000 ; |        |



BaseColors	.byte 	%10111000 ;Score/Time Remaining color (Green)
		.byte   %00011110 ;Drive Head color (Yellow)
	  	.byte 	%00110100 ;Data color (Red)
		.byte 	%11110100 ;Line color (Light Brown)
		.byte 	%11110010 ;Track color (Brown)
		.byte 	%00000000 ;Background color (Black)

MinYPos 	.byte	168	;Top-most scanline of playfield
MaxYPos 	.byte	16	;Bottom-most scanline of playfield


	.byte "J. Grand"

	org $FFFC
	.word Start
	.word Start
